==> 1.1.0.zip: nothing/src/app/plan-display/plan-display.component.ts <==
import { AfterViewInit, ChangeDetectorRef, Component, ElementRef, Input, NgZone, OnInit, ViewChild } from '@angular/core';
import { NgxMoveableComponent, OnDragEnd, OnResize, OnResizeEnd } from 'ngx-moveable';
import { OnDrag } from 'ngx-moveable';
import { ConfigService } from '../_services/config.service';
import { Rect } from '../_models/generated/Rect';
import { Color } from '../color-picker/color-picker.component';
import { ShortcutService } from '../_services/shortcut.service';
import { CarouselComponent } from '../carousel/carousel.component';
import { PlanService } from '../_services/plan.service';
import { Plan, PlanElement } from '../_models/plan';
import { WorldAreaService } from '../_services/world-area.service';
import { WorldArea } from '../_models/world-area';
import { NotesComponent } from '../editor/notes/notes.component';
import { ResizedEvent } from 'angular-resize-event';
import { from } from 'rxjs';
import { open } from '@tauri-apps/api/dialog';
import { OverlayRef } from '@angular/cdk/overlay';
import { OverlayService, StateEvent } from '../_services/overlay.service';
import { appWindow } from '@tauri-apps/api/window';
import { EventsService } from '../_services/events.service';

@Component({
  selector: 'plan-display',
  templateUrl: './plan-display.component.html',
  styleUrls: ['./plan-display.component.scss']
})
export class PlanDisplayComponent implements AfterViewInit, OnInit {
  specialClasses() {
    const waypoint = this.hasWaypoint() ? 'active' : '';
    const trial = this.hasTrial() ? 'trial-active' : '';

    return `${waypoint} ${trial}`;

  }

  @Input() backgroundColor?: String;
  draggable: boolean = true;
  rect?: Rect;
  bounds: any = { "left": 0, "top": 0, "right": 0, "bottom": 0, "position": "css" };


  @ViewChild("moveable") moveable?: NgxMoveableComponent;

  // slides: PlanElement[] = [];
  slideIndex: number = 0;
  zoneSlides?: CarouselComponent<PlanElement>;
  currentSlides?: CarouselComponent<PlanElement>;
  worldAreaMap?: Map<String, WorldArea>;
  settingsOpen: boolean = false;
  init: boolean = false;
  hasAttachedOnce: boolean = false;

  constructor(private events: EventsService, public configService: ConfigService, private cdr: ChangeDetectorRef, private shortcut: ShortcutService, public planService: PlanService, public worldAreaService: WorldAreaService, public overlayService: OverlayService, private zone: NgZone) {
    window.addEventListener("resize", () => {
      this.zone.run(() => {
        this.windowInitHandler()
      })
    });

    // const test = this.events.listen<StateEvent>("OverlayStateChange").subscribe(event => {
    //   if (!this.hasAttachedOnce) {
    //     this.hasAttachedOnce = true;
    //     test.unsubscribe();
    //     if (!event.payload.Hidden)
    //       overlayService.setInteractable();
    //   }

    // });

    appWindow.listen("entered", (entered) => {
      if (this.planService.currentPlan) {
        const current = this.planService.currentPlan.current;
        const length = this.planService.currentPlan.plan.length;
        if (current + 1 < length) {
          if (entered.payload === this.planService.currentPlan.plan[current + 1].area_key) {
            this.zone.run(() => this.next());
          }
        }
      }
    });

    this.planService.getPreviousPlans();
  }

  windowInitHandler() {
    if (window.innerWidth > 0) {
      this.ngAfterViewInit();
    }
  }

  ngOnInit() {
    this.worldAreaService.getWorldAreas().subscribe(a => this.worldAreaMap = a);

  }

  abs(v: number) {
    return Math.abs(v);
  }

  transform() {
    return `translate(${this.rect!.x}px, ${this.rect!.y}px)`;
  }

  width() {
    return `${this.rect!.width}px`;
  }

  height() {
    return `${this.rect!.height}px`;
  }

  hasWaypoint(key?: string): boolean {
    if (!key) {
      key = this.planService.currentPlan!.plan[this.planService.currentPlan!.current].area_key;
    }
    const world_area = this.worldAreaMap?.get(key);
    return world_area!.has_waypoint;
  }

  hasTrial(key?: string): boolean {
    if (!key) {
      key = this.planService.currentPlan!.plan[this.planService.currentPlan!.current].area_key;
    }

    return this.worldAreaService.hasTrial(key);
  }

  ngAfterViewInit(): void {
    if (window.innerWidth > 0) {
      const cfgRect = this.configService.config.initialPlanWindowPosition;

      this.rect = {
        x: cfgRect.x * window.innerWidth,
        y: cfgRect.y * window.innerHeight,
        width: cfgRect.width * window.innerWidth,
        height: cfgRect.height * window.innerHeight,
      }
      this.moveable?.updateRect();

      setTimeout(() => this.cdr.detectChanges(), 0);
      this.init = true;
    }
  }

  onDrag(e: OnDrag) {
    this.rect!.x = e.translate[0];
    this.rect!.y = e.translate[1];
  }

  onDragEnd(e: OnDragEnd) {
    this.saveRect();
  }

  onResize(e: OnResize) {
    this.rect!.width = e.width;
    this.rect!.height = e.height;
    this.onDrag(e.drag);
  }

  onResizeEnd(e: OnResizeEnd) {
    this.saveRect();
  }

  saveRect() {
    const toCfgRect = this.rect!;
    this.configService.config.initialPlanWindowPosition = {
      x: toCfgRect.x / window.innerWidth,
      y: toCfgRect.y / window.innerHeight,
      width: toCfgRect.width / window.innerWidth,
      height: toCfgRect.height / window.innerHeight,
    }
  }

  registerZoneSlides(carousel: CarouselComponent<PlanElement>) {
    this.zoneSlides = carousel;
    this.zoneSlides.setIndex(this.slideIndex);
  }
  registerCurrentSlides(carousel: CarouselComponent<PlanElement>) {
    this.currentSlides = carousel;
    this.currentSlides.setIndex(this.slideIndex);

    if (this.currentSlides) {
      this.shortcut.register(this.configService.config.prev, this.prev.bind(this));
      this.shortcut.register(this.configService.config.next, this.next.bind(this));
    }
  }

  next() {
    this.planService.currentPlan!.next();
    this.currentSlides?.next();
    this.zoneSlides?.next();
  }

  prev() {
    this.planService.currentPlan!.prev();
    this.currentSlides?.prev();
    this.zoneSlides?.prev();
  }

  setIndex(index: number) {
    this.slideIndex = index;
    if (this.currentSlides) {
      this.currentSlides.current = index;

    }
    if (this.zoneSlides) {
      this.zoneSlides.current = index;
    }
  }

  setPrevious(plan: string) {
    this.planService.loadPrevious(plan).subscribe(plan => {
      this.setIndex(plan.current);
    });
  }

  onResizeNote(noteSlide: NotesComponent) {
    if (!noteSlide.ref) { return; }

    let bounds = noteSlide.ref.nativeElement.getBoundingClientRect();
    const children = noteSlide.ref.nativeElement.children;

    let sumWidth = 0;
    let sumHeight = 0;
    for (let child of children) {
      const c = child.getBoundingClientRect();
      sumWidth += c.width;
      sumHeight += c.height;
    }

    const scale = Math.min(
      sumWidth / bounds.width,
      sumHeight / bounds.height,
    )

    for (let child of children) {
      child.style.transform = `scale(1, ${scale})`;
    }

  }

  settingsClick(event: any) {
    this.settingsOpen = !this.settingsOpen;
    event.stopPropagation();
  }

  openDialog() {
    from(open({
      multiple: false,
      filters: [
        {
          name: "JSON (.json)",
          extensions: ['json']
        }
      ]
    })).subscribe(file => {
      if (file) {
        this.planService.loadPlan(file as string).subscribe(plan => {
          if (plan) {
            this.settingsOpen = false;
          }
        });
      }
    });
  }

  loadBasePlan() {
    this.planService.loadBasePlan().subscribe(plan => {
      this.planService.currentPlan = new Plan(plan);
    })
  }

  enumeratedPlans() {

  }

  onScroll(event: WheelEvent) {
    if (event.deltaY < 0) {
      this.prev();
    } else {
      this.next();
    }
  }
}

==> 1.4.0.zip: nothing/src/app/plan-display/plan-display.component.ts <==
import { AfterViewInit, ChangeDetectorRef, Component, Input, NgZone, OnInit, ViewChild } from '@angular/core';
import { NgxMoveableComponent, OnDragEnd, OnResize, OnResizeEnd } from 'ngx-moveable';
import { OnDrag } from 'ngx-moveable';
import { ConfigService } from '../_services/config.service';
import { Rect } from '../_models/generated/Rect';
import { ShortcutService } from '../_services/shortcut.service';
import { CarouselComponent } from '../carousel/carousel.component';
import { PlanService } from '../_services/plan.service';
import { Plan, PlanElement } from '../_models/plan';
import { WorldAreaService } from '../_services/world-area.service';
import { WorldArea } from '../_models/world-area';
import { from } from 'rxjs';
import { open } from '@tauri-apps/api/dialog';
import { OverlayService } from '../_services/overlay.service';
import { appWindow } from '@tauri-apps/api/window';
import { EventsService } from '../_services/events.service';

@Component({
  selector: 'plan-display',
  templateUrl: './plan-display.component.html',
  styleUrls: ['./plan-display.component.scss']
})
export class PlanDisplayComponent implements AfterViewInit, OnInit {
  @Input() backgroundColor?: String;
  draggable: boolean = true;
  rect?: Rect;
  bounds: any = { "left": 0, "top": 0, "right": 0, "bottom": 0, "position": "css" };
  @ViewChild("moveable") moveable?: NgxMoveableComponent;

  slideIndex: number = 0;
  zoneSlides?: CarouselComponent<PlanElement>;
  currentSlides?: CarouselComponent<PlanElement>;
  worldAreaMap?: Map<String, WorldArea>;
  settingsOpen: boolean = false;
  init: boolean = false;
  hasAttachedOnce: boolean = false;

  constructor(private events: EventsService, public configService: ConfigService, private cdr: ChangeDetectorRef, private shortcut: ShortcutService, public planService: PlanService, public worldAreaService: WorldAreaService, public overlayService: OverlayService, private zone: NgZone) {
    window.addEventListener("resize", () => {
      this.zone.run(() => {
        this.windowInitHandler()
      })
    });

    appWindow.listen("entered", (entered) => {
      if (this.planService.currentPlan) {
        const current = this.planService.currentPlan.current;
        const length = this.planService.currentPlan.plan.length;
        if (current + 1 < length) {
          if (entered.payload === this.planService.currentPlan.plan[current + 1].area_key) {
            this.zone.run(() => this.next());
          }
        }
      }
    });

    this.planService.getPreviousPlans();
  }

  windowInitHandler() {
    if (window.innerWidth > 0) {
      this.ngAfterViewInit();
    }
  }

  ngOnInit() {
    this.worldAreaService.getWorldAreas().subscribe(a => this.worldAreaMap = a);

  }

  abs(v: number) {
    return Math.abs(v);
  }

  transform() {
    return `translate(${this.rect!.x}px, ${this.rect!.y}px)`;
  }

  width() {
    return `${this.rect!.width}px`;
  }

  height() {
    return `${this.rect!.height}px`;
  }

  hasWaypoint(key?: string): boolean {
    if (!key) {
      key = this.planService.currentPlan!.plan[this.planService.currentPlan!.current].area_key;
    }
    const world_area = this.worldAreaMap?.get(key);
    return world_area!.has_waypoint;
  }

  hasTrial(key?: string): boolean {
    if (!key) {
      key = this.planService.currentPlan!.plan[this.planService.currentPlan!.current].area_key;
    }

    return this.worldAreaService.hasTrial(key);
  }

  ngAfterViewInit(): void {
    if (window.innerWidth > 0) {
      const cfgRect = this.configService.config.initialPlanWindowPosition;

      this.rect = {
        x: cfgRect.x * window.innerWidth,
        y: cfgRect.y * window.innerHeight,
        width: cfgRect.width * window.innerWidth,
        height: cfgRect.height * window.innerHeight,
      }
      this.moveable?.updateRect();

      setTimeout(() => this.cdr.detectChanges(), 0);
      this.init = true;
    }
  }

  onDrag(e: OnDrag) {
    this.rect!.x = e.translate[0];
    this.rect!.y = e.translate[1];
  }

  onDragEnd(e: OnDragEnd) {
    this.saveRect();
  }

  onResize(e: OnResize) {
    this.rect!.width = e.width;
    this.rect!.height = e.height;
    this.onDrag(e.drag);
  }

  onResizeEnd(e: OnResizeEnd) {
    this.saveRect();
  }

  saveRect() {
    const toCfgRect = this.rect!;
    this.configService.config.initialPlanWindowPosition = {
      x: toCfgRect.x / window.innerWidth,
      y: toCfgRect.y / window.innerHeight,
      width: toCfgRect.width / window.innerWidth,
      height: toCfgRect.height / window.innerHeight,
    }
  }

  registerZoneSlides(carousel: CarouselComponent<PlanElement>) {
    this.zoneSlides = carousel;
    this.zoneSlides.setIndex(this.slideIndex);
  }

  registerCurrentSlides(carousel: CarouselComponent<PlanElement>) {
    this.currentSlides = carousel;
    this.currentSlides.setIndex(this.slideIndex);

    if (this.currentSlides) {
      this.shortcut.register(this.configService.config.prev, this.prev.bind(this));
      this.shortcut.register(this.configService.config.next, this.next.bind(this));
    }
  }

  next() {
    this.planService.currentPlan!.next();
    this.currentSlides?.next();
    this.zoneSlides?.next();
  }

  prev() {
    this.planService.currentPlan!.prev();
    this.currentSlides?.prev();
    this.zoneSlides?.prev();
  }

  setIndex(index: number) {
    this.slideIndex = index;
    if (this.currentSlides) {
      this.currentSlides.setIndex(index);
    }
    if (this.zoneSlides) {
      this.zoneSlides.setIndex(index);
    }
  }

  setPrevious(plan: string) {
    this.planService.loadPrevious(plan).subscribe(plan => {
      setTimeout(() => this.setIndex(plan.current), 0);
    });
  }

  settingsClick(event: any) {
    this.settingsOpen = !this.settingsOpen;
    event.stopPropagation();
  }

  openDialog() {
    from(open({
      multiple: false,
      filters: [
        {
          name: "JSON (.json)",
          extensions: ['json']
        }
      ]
    })).subscribe(file => {
      if (file) {
        this.planService.loadPlan(file as string).subscribe(plan => {
          if (plan) {
            this.settingsOpen = false;
          }
        });
      }
    });
  }

  loadBasePlan() {
    this.planService.loadBasePlan().subscribe(plan => {
      this.planService.currentPlan = new Plan(plan);
      if (this.zoneSlides) {
        this.zoneSlides.setIndex(0);
      }
      if (this.currentSlides) {
        this.currentSlides.setIndex(0);
      }
    })
  }

  onScroll(event: WheelEvent) {
    if (event.deltaY < 0) {
      this.prev();
    } else {
      this.next();
    }
  }

  specialClasses() {
    const waypoint = this.hasWaypoint() ? 'active' : '';
    const trial = this.hasTrial() ? 'trial-active' : '';
    return `${waypoint} ${trial}`;
  }

  clampedOffset(): number {
    return Math.min(this.configService.config.numVisible - 1, this.configService.config.offset);
  }

  zonesStyle() {
    return {
      'min-height': `${this.configService.config.numVisible * 18}px`,
      'max-height': `${this.configService.config.numVisible * 40}px`
    }
  }
}
